function [p xi] = ccghc(x,w,E,u,l)
% ccghc - cost constrained geometric Huffman coding
%   p = ccghc(x,w,E) returns the dyadic probability mass function (pmf) that
%   minimizes the Kullback-Leibler distance kldiv(p,x) subject to the
%   average cost constraint w'*p <= E among all dyadic pmfs p.
%
%   [p xi] = ccghc(x,w,E) returns the optimal scaling of the cost constraint,
%   i.e., p minimizes kldiv(p,x) + xi*w'*p.
%
%   ccghc tries to guess upper and lower bounds on the scaling factor xi.
%   optionally, upper bound u and lower bound l can be passed to ccghc by
%   calling ccghc(x,w,E,u,l).
%
%   x has to be a n-by-1 matrix with nonnegative entries.
%   w has to be a n-by-1 matrix with positive entries.
%   E has to be a feasible positive scalar.
%   u and l have to ensure u > xi > l.
%
%   Example:
%     [p xi] = ccghc([0.4 0.4 0.2]',[1 2 3]',1.5) returns:
%       p =
%
%          0.50000
%          0.50000
%          0.00000
%
%       xi =  0.69315
%
%   See also ghc, nghc, kldiv
%
% tested for octave 3.4.0, Matlab R2010b
% 29 June 2011
% (c) Georg Böcherer
% email: geborg@gmail.com
if(nargin < 4)
    u = 100*eps;
    n=1;
    while(1)
        p = ghc(x.*exp(-u*w));
        if(w'*p<E)
            break;
        else
            u=2*u;
        end
        n=n+1;
    end
end

if(nargin < 5)
    l = -100*eps;
    n=1;
    while(1)
        p = ghc(x.*exp(-l*w));
        if(w'*p > E)
            break;
        else
            l = 2*l;
        end
        n=n+1;
    end
end
my_eps = 10^(-7);
n=1;

while(1)
    xi = (l+u)/2;
    p = ghc(x.*exp(-xi*w));
    if(w'*p>E)
        l = xi;
    else
        u = xi;
    end
    if(abs(u-l)<my_eps)
        break;
    end
    n=n+1;
end
p = ghc(x.*exp(-u*w));

